# il faut connaître les bibliothèques à importer
import matplotlib.pyplot as plt
import numpy as np
import numpy.random as rd

# On se limite au cas des distributions uniformes
# Dans ce cas, le résultat du mesurage de la grandeur X sera présenté sous la forme d'une liste [xmin,xmax]
# Si, par exemple, on a mesuré une distance comprise entre xmin = 11,3 cm et xmax = 13,5 cm, il suffit de rentrer X = [11.3,13.5]
# Si le constructeur fournit une valeur x_moy +- Delta_x, il suffit de rentrer X = [x_moy -Delta_x,x_moy + Delta_x]
# Si c'est la valeur moyenne x_moy et l'incertitude-type u(X) qui sont connues, on rentre X = [xmoy - u(X)*3**0.5, x_moy + u(X)*3**0.5]

# On a ainsi rentré X1, X2,...,Xn, et on cherche la valeur moyenne de X = f(X1,X2,...,Xn) et son incertitude-type u(X). La fonction f est une fonction de n variables

N = 1000000 # Simuler N = un million d'expériences

def propagation_monte_carlo(*arg): # la fonction admet un nombre n quelconque d'arguments
    L = [] # on crée une liste vide
    for elt in arg:
        L.append(rd.uniform(elt[0],elt[1],N))
        # L contient n tableaux numpy donnant pour chaque variable N valeurs aléatoires conformes à la distribution uniforme de la variable
    X = f(*L) # X contient N valeurs obtenues en appliquant f aux éléments de L
    # X[0] est par exemple égal à f(X1[0],X2[0],...,Xn[0])
    # On a ainsi créé N valeurs de X qui permettent d'obtenir x_moy et u(X)
    mu = np.mean(X) # valeur moyenne de X
    sigma = np.std(X, 0, ddof = 1) # incertitude-type sur X
    plt.figure()
    plt.hist(X,100) # histogramme de X, avec 100 intervalles
    plt.show()
    return mu,sigma # on renvoie x_moy et u(X)

# Exemple : tension d'entrée ue aux bornes de R en série avec C, tension de sortie us aux bornes de R. Le déphasage théorique entre us et ue est phi = arctan(1/(2*pi*R*C)).
# Le constructeur donne une tolérance de 1% sur R et de 0,5% sur C. Les valeurs utilisées sont 100 ohms et 1 µF. L'incertitude relative sur la fréquence F = 1000 Hz est très inférieure à celles sur R et C, donc on ne la prend pas en compte
F = 1E3
def f(R,C): # valeur théorique du déphasage, transformée en degrés
    return np.arctan(1/(2*np.pi*F*R*C))*180/np.pi

R = [99,101]
C = [1E-6-0.005*1E-6,1E-6+0.005*1E-6]

print(propagation_monte_carlo(R,C))

# On peut comparer avec la méthode analytique, qui donne ici le bon résulat car, comme souvent, les incertitudes relatives sont faibles

# On obtient avec la formule théorique phi_th = 57,86° avec u(phi_th) = 0,17°
# Il reste à mesurer phi_exp à l'oscilloscope et à l'accompagner de u(phi_exp)
# La compatibilité entre les valeurs expérimentale et théorique passe par le calcul du Z-score:
# Z = (phi_exp - phi_th) / ((u(phi_exp)**2 + u(phi_th)**2))**0.5
# un Z-score qui est en valeur absolue supérieur à 2 invalide la compatibilité. Si les mesures ont été bien faites et les incertitudes bien évaluées, alors c'est que la théorie n'est pas adaptée (hypothèses non vérifiées, phénomènes pas pris en compte, etc.)


